###############################################################################
#	Filename:	M4Complex.py
#	
#	Confidential and Proprietary, Copyright 2002 by Totally Games
#	
#	A fairly complicated mission.
#	
#	Created:	2/4/2002	- Erik Novales (ripped off of E2M6 because Jess
#							  has good style :P )
###############################################################################
import App
import loadspacehelper
import MissionLib

# Mission state enumeration. This helps make it easier to keep track of where
# the user is in the mission.
MS_MISSION_START			= 0
MS_ARRIVED_BIRANU2			= 1
MS_ARRIVED_BIRANU1			= 2
MS_SCANNED_FREIGHTER		= 3

g_eMissionState				= MS_MISSION_START
g_pMissionTGL = None		# string database for the mission

###############################################################################
#	PreLoadAssets()
#	
#	This is called once, at the beginning of the mission before Initialize()
#	to allow us to add instances of ships we'll use. This is not mandatory,
#	but will prevent hitching when ships are created.
#	
#	Args:	pMission	- the Mission object
#	
#	Return:	none
###############################################################################
def PreLoadAssets(pMission):
	# Pre-create one Sovereign ship, three Galors, an Akira, and a freighter.
	loadspacehelper.PreloadShip("Sovereign", 1)
	loadspacehelper.PreloadShip("Galor", 3)
	loadspacehelper.PreloadShip("Akira", 1)
	loadspacehelper.PreloadShip("Freighter", 1)
	
################################################################################
#	Initialize()
#	
#	Called once when mission loads to initialize mission.
#	
#	Args: 	pMission	- the mission object
#	
#	Return: None
################################################################################
def Initialize(pMission):
	global g_pMissionTGL
	g_pMissionTGL = App.g_kLocalizationManager.Load("data/TGL/Tutorial/Episode/M4Complex.tgl")

	global g_eMissionState
	g_eMissionState = MS_MISSION_START

	# Specify (and load if necessary) the player's bridge. This is only
	# necessary for the first mission in the game, unless you're switching
	# bridges.
	import LoadBridge
	LoadBridge.Load("SovereignBridge")

	# Create the regions for this mission
	CreateRegions()

	# Create the other viewscreen sets.
	CreateViewscreenSets()
	
	# Create the starting objects
	CreateStartingObjects(pMission)

	# Setup object AI.
	SetupAI()

	# Setup event handlers.
	SetupEventHandlers()

	# Mission briefing()
	MissionBriefing()
	
################################################################################
#	CreateRegions()
#
#	Creates the regions we'll be using in this mission.  Also loads in any
#	mission placement files we need.
#
#	Args:	None
#
#	Return:	None
################################################################################
def CreateRegions():
	# Biranu
	import Systems.Biranu.Biranu
	pBiranuMenu	= Systems.Biranu.Biranu.CreateMenus()
	pBiranu1Set = MissionLib.SetupSpaceSet("Systems.Biranu.Biranu1")
	pBiranu2Set = MissionLib.SetupSpaceSet("Systems.Biranu.Biranu2")

	# Starbase 12
	import Systems.Starbase12.Starbase
	pSB12Menu	= Systems.Starbase12.Starbase.CreateMenus()
	pSB12Set	= MissionLib.SetupSpaceSet("Systems.Starbase12.Starbase12")
	
	# Add our custom placement objects for this mission.
	import M4Biranu2_P
	M4Biranu2_P.LoadPlacements(pBiranu2Set.GetName())

	import M4Biranu1_P
	M4Biranu1_P.LoadPlacements(pBiranu1Set.GetName())

	import M4SB12_P
	M4SB12_P.LoadPlacements(pSB12Set.GetName())

	# Bridge placements
	import M4EBridge_P
	M4EBridge_P.LoadPlacements("bridge")

###############################################################################
#	CreateViewscreenSets()
#	
#	Create any other sets needed for the mission.
#	
#	Args:	none
#	
#	Return:	none
###############################################################################
def CreateViewscreenSets():
	# Freighter bridge and captain.
	pSet	= MissionLib.SetupBridgeSet("FreighterSet", "data/Models/Sets/Cardassian/cardbridge.NIF", -30, 65, -1.55)
	pCapt	= MissionLib.SetupCharacter("Bridge.Characters.Korbus", "FreighterSet")

################################################################################
#	CreateStartingObjects()
#
#	Creates all the objects that exist at the beginning of the mission and sets
# 	up the affiliations for all objects that will occur in the mission.
#
#	Args:	pMission	- The mission object.
#
#	Return:	None
################################################################################
def CreateStartingObjects(pMission):
	# get the sets we need
	pSB12Set		= App.g_kSetManager.GetSet("Starbase12")
	pBiranu1Set		= App.g_kSetManager.GetSet("Biranu1")
	
	# Create the ships that exist at mission start.
	pPlayer			= MissionLib.CreatePlayerShip("Sovereign", pSB12Set, "player", "Player Start")
	pSB12			= loadspacehelper.CreateShip("FedStarbase", pSB12Set, "Starbase 12", "Starbase12 Location")
	pGeronimo		= loadspacehelper.CreateShip("Akira", pSB12Set, "Geronimo", "Geronimo Start")
	pFreighter		= loadspacehelper.CreateShip("Freighter", pBiranu1Set, "Mystery Freighter", "Freighter Start")

	# Pre-damage the freighter, both visually and system-wise.
	MissionLib.DamageShip("Mystery Freighter", 0.6, 0.9)
	import FreighterDamage
	FreighterDamage.AddDamage(pFreighter)

	# Setup ship affiliations.
	pFriendlies = pMission.GetFriendlyGroup()
	pFriendlies.AddName(pPlayer.GetName())
	pFriendlies.AddName(pGeronimo.GetName())

	pNeutrals = pMission.GetNeutralGroup()
	pNeutrals.AddName(pFreighter.GetName())

	# You can add things to groups even if they don't exist yet, because
	# they are referenced by name.
	pEnemies = pMission.GetEnemyGroup()
	for i in range(3):
		pEnemies.AddName("Galor " + str(i + 1))

###############################################################################
#	SetupAI()
#	
#	Sets up object AI.
#	
#	Args:	none
#	
#	Return:	none
###############################################################################
def SetupAI():
	pSet = App.g_kSetManager.GetSet("Starbase12")
	pGame = App.Game_GetCurrentGame()

	pGeronimo = App.ShipClass_GetObject(pSet, "Geronimo")

	import GeronimoAI

	pGeronimo.SetAI(GeronimoAI.CreateAI(pGeronimo))

###############################################################################
#	SetupEventHandlers()
#	
#	Sets up event handlers for this mission.
#	
#	Args:	none
#	
#	Return:	none
###############################################################################
def SetupEventHandlers():
	pGame = App.Game_GetCurrentGame()
	pEpisode = pGame.GetCurrentEpisode()
	pMission = pEpisode.GetCurrentMission()

	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_EXITED_WARP,
		pMission, __name__ + ".HandleExitedWarp")
	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_OBJECT_EXPLODING,
		pMission, __name__ + ".HandleObjectExploding")

	# Special handlers for menu buttons
	# Helm buttons
	import Bridge.BridgeUtils
	pKiskaMenu = Bridge.BridgeUtils.GetBridgeMenu("Helm")
	if (pKiskaMenu != None):
		# Instance handler for warp button
		pWarpButton = Bridge.BridgeUtils.GetWarpButton()
		if (pWarpButton != None):
			pWarpButton.AddPythonFuncHandlerForInstance(App.ET_WARP_BUTTON_PRESSED, __name__ + ".HandleWarpButton")

	pMiguelMenu = Bridge.BridgeUtils.GetBridgeMenu("Science")
	if (pMiguelMenu != None):
		pMiguelMenu.AddPythonFuncHandlerForInstance(App.ET_SCAN,		__name__ + ".HandleScan")

###############################################################################
#	MissionBriefing()
#	
#	Does the mission briefing.
#	
#	Args:	none
#	
#	Return:	none
###############################################################################
def MissionBriefing():
	pSequence = App.TGSequence_Create()

	# Subtitle-only lines will remain until they are brought down. Voice lines
	# go down when the sound ends. Since we are using subtitle-only lines here,
	# we'll add another script action to bring down each line.
	pLine1 = App.SubtitleAction_Create(g_pMissionTGL, "Briefing1")
	pSequence.AddAction(pLine1)
	pLineDown1 = App.TGScriptAction_Create(__name__, "EndAction", pLine1.GetObjID())
	# We use App.TGAction_CreateNull() here because we don't want
	# this action to follow the line -- we want it to just happen 
	# 8 seconds after the sequence starts. Normally, we'd put the action
	# we wanted to follow in that place.
	pSequence.AddAction(pLineDown1, App.TGAction_CreateNull(), 10.0)

	pLine2 = App.SubtitleAction_Create(g_pMissionTGL, "Briefing2")
	pSequence.AddAction(pLine2, pLine1, 1.0)
	pLineDown2 = App.TGScriptAction_Create(__name__, "EndAction", pLine2.GetObjID())
	pSequence.AddAction(pLineDown2, pLineDown1, 15.0)

	pLine3 = App.SubtitleAction_Create(g_pMissionTGL, "Briefing3")
	pSequence.AddAction(pLine3, pLine2, 1.0)
	pLineDown3 = App.TGScriptAction_Create(__name__, "EndAction", pLine3.GetObjID())
	pSequence.AddAction(pLineDown3, pLineDown2, 10.0)

	# Play the sequence.
	pSequence.Play()

###############################################################################
#	EndAction(pAction, idActionToEnd)
#	
#	Ends the action with the specified ID. Used above to remove subtitle-only
#	lines.
#	
#	Args:	pAction			- this action, passed in automatically
#			idActionToEnd	- the object ID of the action to end
#	
#	Return:	zero, for normal action end
###############################################################################
def EndAction(pAction, idActionToEnd):
	pAction = App.TGAction_Cast(App.TGObject_GetTGObjectPtr(idActionToEnd))

	if pAction:
		pAction.Completed()	# Finish the action.

	return 0

###############################################################################
#	HandleExitedWarp(pMission, pEvent)
#	
#	Event handler for when an object is done warping.
#	
#	Args:	pMission	- the mission (the object specified in the event
#						  handler)
#			pEvent		- the event
#	
#	Return:	none
###############################################################################
def HandleExitedWarp(pMission, pEvent):
	# Check if the object is the player.
	pShip = App.ShipClass_Cast(pEvent.GetSource())
	pPlayer = App.Game_GetCurrentPlayer()

	if (pShip != None) and (pPlayer != None) and (pShip.GetObjID() == pPlayer.GetObjID()):
		# It's the player.
		global g_eMissionState
		pSet = pShip.GetContainingSet()
		if not pSet:
			pMission.CallNextHandler(pEvent)
			return

		# If the player is in the MS_MISSION_START state and they
		# are entering Biranu 2, proceed to the next state of the mission.
		if (g_eMissionState == MS_MISSION_START) and (pSet.GetName() == "Biranu2"):
			# Move to the next state in the mission.
			g_eMissionState = MS_ARRIVED_BIRANU2

			# Play the cutscene showing the ships warping in.
			Biranu2Cutscene()

		if (g_eMissionState == MS_ARRIVED_BIRANU2) and (pSet.GetName() == "Biranu1"):
			# Move to the next state in the mission.
			g_eMissionState = MS_ARRIVED_BIRANU1

			# Play the cutscene after you enter the area.
			Biranu1Cutscene()
		
	pMission.CallNextHandler(pEvent)

###############################################################################
#	HandleObjectExploding(pMission, pEvent)
#	
#	Handler called whenever something is dying.
#	
#	Args:	pMission	- the mission (the object specified in the event
#						  handler)
#			pEvent		- the event
#	
#	Return:	none
###############################################################################
def HandleObjectExploding(pMission, pEvent):
	# If the player is in Biranu 2, check if it was the last enemy ship
	# exploding.
	pPlayer = App.Game_GetCurrentPlayer()
	pGame = App.Game_GetCurrentGame()
	pEpisode = pGame.GetCurrentEpisode()
	pMission = pEpisode.GetCurrentMission()

	if (pPlayer != None) and (g_eMissionState == MS_ARRIVED_BIRANU2):
		pSet = pPlayer.GetContainingSet()
		pShip = App.ShipClass_Cast(pEvent.GetDestination())
		pEnemies = pMission.GetEnemyGroup()

		# Make sure it was an enemy that was blowing up.
		if (pShip != None) and (pEnemies.IsNameInGroup(pShip.GetName()) == 0):
			pMission.CallNextHandler(pEvent)
			return

		for i in range(3):
			pShip = App.ShipClass_Cast(App.ShipClass_GetObject(pSet, "Galor " + str(i + 1)))
			if (pShip != None) and (not pShip.IsDying()) and (not pShip.IsDead()):
				pMission.CallNextHandler(pEvent)
				return

		# No live enemies left. Play a little sequence.
		pBridge = App.g_kSetManager.GetSet("bridge")
		pFelix = App.CharacterClass_GetObject(pBridge, "Tactical")
		pMiguel = App.CharacterClass_GetObject(pBridge, "Science")

		pSequence = App.TGSequence_Create()
		pSequence.AppendAction(App.CharacterAction_Create(pFelix, 
			App.CharacterAction.AT_SAY_LINE, "FelixEnDes", None, 1, pFelix.GetDatabase()))
		pSequence.AppendAction(App.CharacterAction_Create(pMiguel, 
			App.CharacterAction.AT_SAY_LINE, "gs038", "Captain", 1, pMiguel.GetDatabase()))
		pSequence.Play()

	pMission.CallNextHandler(pEvent)

###############################################################################
#	HandleWarpButton(pWarpButton, pEvent)
#	
#	Handler called when the user clicks on the warp button. To prevent warping,
#	don't call the next handler.
#	
#	Args:	pWarpButton	- the warp button
#			pEvent		- the event
#	
#	Return:	none
###############################################################################
def HandleWarpButton(pWarpButton, pEvent):
	# If we're in Biranu 2, and there are still enemies around, don't let the
	# player warp.
	pPlayer = App.Game_GetCurrentPlayer()
	if not pPlayer:
		pWarpButton.CallNextHandler(pEvent)
		return

	pSet = pPlayer.GetContainingSet()
	if not pSet:
		pWarpButton.CallNextHandler(pEvent)
		return

	if (pSet.GetName() == "Biranu2") and (g_eMissionState == MS_ARRIVED_BIRANU2):
		# Check if there are still enemies in Biranu 2. If so, then don't let
		# the player warp. Otherwise, it's OK.
		for i in range(3):
			pShip = App.ShipClass_Cast(App.ShipClass_GetObject(pSet, "Galor " + str(i + 1)))
			if (pShip != None) and (not pShip.IsDying()) and (not pShip.IsDead()):
				# There's an enemy still here. The player can't warp.
				pBridge = App.g_kSetManager.GetSet("bridge")
				pSaffi = App.CharacterClass_GetObject(pBridge, "XO")
				pAction = App.CharacterAction_Create(pSaffi, 
					App.CharacterAction.AT_SAY_LINE, "WarpStop2", "Captain", 1, pSaffi.GetDatabase())
				pAction.Play()
				return


	pWarpButton.CallNextHandler(pEvent)

###############################################################################
#	HandleScan(pMenu, pEvent)
#	
#	Handler for when Scan is selected.
#	
#	Args:	pMenu	- the science menu
#			pEvent	- the event
#	
#	Return:	none
###############################################################################
def HandleScan(pMenu, pEvent):
	# If the player is in Biranu 1, and is trying to scan the freighter, make
	# sure that they're close enough before proceeding.
	pPlayer = App.Game_GetCurrentPlayer()
	if not pPlayer:
		pMenu.CallNextHandler(pEvent)
		return

	pSensors = pPlayer.GetSensorSubsystem()

	pSet = pPlayer.GetContainingSet()

	if (g_eMissionState == MS_ARRIVED_BIRANU1) and (pSet) and (pSet.GetName() == "Biranu1"):
		eScanType = pEvent.GetInt()

		pTarget = App.ShipClass_Cast(pPlayer.GetTarget())
		pObject = App.ObjectClass_Cast(pEvent.GetSource())
		pFreighter = App.ShipClass_GetObject(pSet, "Mystery Freighter")

		if pObject:
			pTarget = pObject

		if (not pFreighter) or (not pTarget):
			pMenu.CallNextHandler(pEvent)
			return

		if ((eScanType == App.CharacterClass.EST_SCAN_AREA) or
			((eScanType == App.CharacterClass.EST_SCAN_OBJECT) and 
			 (pTarget.GetObjID() == pFreighter.GetObjID()))):
			# Check distance to freighter. If they're within 20km, then continue
			# with the mission.
			fDistance = MissionLib.GetDistance(pTarget, pPlayer)
			if (App.UtopiaModule_ConvertGameUnitsToKilometers(fDistance) < 20.0):
				global g_eMissionState
				g_eMissionState = MS_SCANNED_FREIGHTER

				ScanCutscene()
				# Don't need to call next handler.
				return
			else:
				# Tell the player they're too far away.
				pBridge = App.g_kSetManager.GetSet("bridge")
				pMiguel = App.CharacterClass_GetObject(pBridge, "Science")
				pAction = App.CharacterAction_Create(pMiguel, App.CharacterAction.AT_SAY_LINE, "gs032", None, 0, pMiguel.GetDatabase())
				pAction.Play()
				return

	pMenu.CallNextHandler(pEvent)

###############################################################################
#	Biranu2Cutscene()
#	
#	Creates the sequence for when the player first enters Biranu 2. Shows
#	three Galors warping in to attack the player.
#	
#	Args:	none
#	
#	Return:	none
###############################################################################
def Biranu2Cutscene():
	pBiranu2Set = App.g_kSetManager.GetSet("Biranu2")
	pBridge = App.g_kSetManager.GetSet("bridge")
	pFelix = App.CharacterClass_GetObject(pBridge, "Tactical")

	# Create the Galors, and have them warp in. The extra parameter at the end
	# indicates that they should warp in (from nowhere). We won't assign their
	# AI just yet, because having them attack the player during the cutscene
	# would be unfair.
	pGalor1 = loadspacehelper.CreateShip("Galor", pBiranu2Set, "Galor 1", "Galor 1 Start", 1)
	pGalor2 = loadspacehelper.CreateShip("Galor", pBiranu2Set, "Galor 2", "Galor 2 Start", 1)
	pGalor3 = loadspacehelper.CreateShip("Galor", pBiranu2Set, "Galor 3", "Galor 3 Start", 1)

	pSeq = App.TGSequence_Create()

	# Start the cutscene.
	pSeq.AppendAction(App.TGScriptAction_Create("MissionLib", "StartCutscene"))
	# Create a cutscene camera for Biranu 2.
	pSeq.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "CutsceneCameraBegin", "Biranu2"))
	# Make the cutscene camera watch the Galors warping in.
	pSeq.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "PlacementWatch", "Biranu2", 
					  "Galor 3 Start", "Cutscene Camera Placement"))
	# Change us so we're rendering Biranu 2 instead of whatever else.
	pSeq.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "ChangeRenderedSet", "Biranu2"))
	# Have Felix announce the incoming enemies.
	pSeq.AppendAction(App.CharacterAction_Create(pFelix, App.CharacterAction.AT_SAY_LINE, "Incoming3", None, 0, pFelix.GetDatabase()))
	pSeq.AppendAction(App.CharacterAction_Create(pFelix, App.CharacterAction.AT_SAY_LINE, "gt031", None, 0, pFelix.GetDatabase()))

	# Kill the cutscene camera.
	pSeq.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "CutsceneCameraEnd", "Biranu2"), 4.0)
	# Move back to the bridge.
	pSeq.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "ChangeRenderedSet", "bridge"))
	# End the cutscene.
	pSeq.AppendAction(App.TGScriptAction_Create("MissionLib", "EndCutscene"))
	# Assign the AIs for the Galors.
	pSeq.AppendAction(App.TGScriptAction_Create(__name__, "AssignGalorAI"))

	pSeq.Play()

###############################################################################
#	Biranu1Cutscene()
#	
#	Plays the cutscene when the player enters Biranu 1.
#	
#	Args:	none
#	
#	Return:	none
###############################################################################
def Biranu1Cutscene():
	pBridge = App.g_kSetManager.GetSet("bridge")
	pMiguel = App.CharacterClass_GetObject(pBridge, "Science")

	pSeq = App.TGSequence_Create()

	pSeq.AppendAction(App.TGScriptAction_Create("MissionLib", "StartCutscene"))
	pSeq.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "CutsceneCameraBegin", "bridge"))

	pSeq.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "ChangeRenderedSet", "bridge"))

	# We use a placement offset watch for looking at characters, so we're
	# looking at their heads and not their bodies.
	pSeq.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "PlacementOffsetWatch", 
		"bridge", "Science", "Science Camera Placement", 0.0, -10.0, pMiguel.GetHeadHeight() * 0.40))
	pSeq.AppendAction(App.CharacterAction_Create(pMiguel, App.CharacterAction.AT_SAY_LINE, "gs025", None, 0, pMiguel.GetDatabase()))
	pSeq.AppendAction(App.CharacterAction_Create(pMiguel, App.CharacterAction.AT_SAY_LINE, "gs035", None, 0, pMiguel.GetDatabase()))
	pSeq.AppendAction(App.TGScriptAction_Create("MissionLib", "SetTarget", "Mystery Freighter"))
	pSeq.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "CutsceneCameraEnd", "bridge"))
	pSeq.AppendAction(App.TGScriptAction_Create("MissionLib", "LookForward", 1))
	pSeq.AppendAction(App.CharacterAction_Create(pMiguel, App.CharacterAction.AT_SAY_LINE, "gs032", None, 0, pMiguel.GetDatabase()))
	pSeq.AppendAction(App.CharacterAction_Create(pMiguel, App.CharacterAction.AT_SAY_LINE, "MiguelNothingToAdd3", None, 0, pMiguel.GetDatabase()))
	pSeq.AppendAction(App.TGScriptAction_Create("MissionLib", "EndCutscene"))

	pSeq.Play()

###############################################################################
#	ScanCutscene()
#	
#	Cutscene that plays when you scan the freighter.
#	
#	Args:	none
#	
#	Return:	none
###############################################################################
def ScanCutscene():
	pPlayer = App.Game_GetCurrentPlayer()
	pBridge = App.g_kSetManager.GetSet("bridge")
	pScience = App.CharacterClass_GetObject(pBridge, "Science")
	pSaffi	= App.CharacterClass_GetObject(pBridge, "XO")
	pSensors = pPlayer.GetSensorSubsystem()

	import Bridge.ScienceCharacterHandlers
	pScanSequence = Bridge.ScienceCharacterHandlers.GetScanSequence()
	pSequence = App.TGSequence_Create()
	pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "StartCutscene"))
	pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "ChangeRenderedSet", "bridge"))
	pSequence.AppendAction(pScanSequence)
	# "On screen."
	pSequence.AppendAction(App.CharacterAction_Create(pScience, App.CharacterAction.AT_SAY_LINE, "gs035", "Captain", 1, pScience.GetDatabase()), pSensors.GetIdentificationTime() + 0.5)
	pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "LookForward", 1))

	# Change the viewscreen to use the freighter camera.
	pFreighterSet = App.g_kSetManager.GetSet("FreighterSet")
	pKorbus = App.CharacterClass_GetObject(pFreighterSet, "Korbus")
	pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "ViewscreenOn", "FreighterSet"))

	# Korbus says hi.
	pSequence.AppendAction(App.CharacterAction_Create(pKorbus, App.CharacterAction.AT_SAY_LINE, "KorbusSir4", None, 0, pKorbus.GetDatabase()))
	pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "ViewscreenOff"))

	pDatabase = App.g_kLocalizationManager.Load("data/TGL/CharacterStatus.tgl")
	pSequence.AppendAction(App.CharacterAction_Create(pScience, App.CharacterAction.AT_SET_STATUS, "Waiting", None, 0, pDatabase))
	App.g_kLocalizationManager.Unload(pDatabase)
	pSequence.AppendAction(App.TGScriptAction_Create("Bridge.BridgeUtils", "EnableScanMenu"))

	pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "EndCutscene"))

	# Do a game over, mentioning the end of the mission.
	pEndAction = App.CharacterAction_Create(pSaffi, App.CharacterAction.AT_SAY_LINE, "GoalComplete", "Captain", 1, pSaffi.GetDatabase())
	pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "GameOver", pEndAction))

	pSequence.Play()


###############################################################################
#	AssignGalorAI(pAction)
#	
#	Script action that assigns the attack AI for the Galors that warp into
#	Biranu 2.
#	
#	Args:	pAction		- this action
#	
#	Return:	zero, to indicate normal action termination
###############################################################################
def AssignGalorAI(pAction):
	pBiranu2Set = App.g_kSetManager.GetSet("Biranu2")
	pGalor1 = App.ShipClass_GetObject(pBiranu2Set, "Galor 1")
	pGalor2 = App.ShipClass_GetObject(pBiranu2Set, "Galor 2")
	pGalor3 = App.ShipClass_GetObject(pBiranu2Set, "Galor 3")

	import GalorAI
	pGalor1.SetAI(GalorAI.CreateAI(pGalor1))
	pGalor2.SetAI(GalorAI.CreateAI(pGalor2))
	pGalor3.SetAI(GalorAI.CreateAI(pGalor3))
	
	return 0

################################################################################
#	Terminate()
#	
#	Called when mission ends to free resources
#	
#	Args: pMission - the mission object
#	
#	Return: None
################################################################################
def Terminate(pMission):
	# Clear out all the systems in the set course menu.
	App.SortedRegionMenu_ClearSetCourseMenu()

	global g_pMissionTGL
	if g_pMissionTGL:
		App.g_kLocalizationManager.Unload(g_pMissionTGL)
		g_pMissionTGL = None
